import { Component } from '@angular/core';
import { RouterModule } from '@angular/router';
import { NgFor,NgIf } from '@angular/common';
import { ActivatedRoute } from '@angular/router';
import { UserMenuComponent } from '../user-menu/user-menu.component'; 
import { SudokuService } from '../../services/sudoku-service';
import { Block } from '@angular/compiler';

@Component({
  selector: 'app-sudoku',
  standalone: true,
  imports: [RouterModule, NgFor, NgIf,UserMenuComponent],
  templateUrl: './sudoku.component.html',
  styleUrl: './sudoku.component.css'
})
export class SudokuComponent {
  grille: number[][] = [];
  nbIndice: number = 0;
  haveWin: boolean = false;
  nbEachNumber:number[] = [0,0,0,0,0,0,0,0,0];
  lastInputClicked: HTMLInputElement | undefined;

  constructor(protected sudokuService: SudokuService, private activatedRoute: ActivatedRoute) {
  }

  ngOnInit() {
    const difficulty = this.activatedRoute.snapshot.params["difficulty"];
    this.Request(difficulty);
  }
  public publish() {
    this.resetError();
    this.haveWin = this.sudokuService.publishGame(this.getGrid(),"pas test",this.nbIndice)
    if (this.haveWin) return;
    this.corriger();
  }

  public corriger() {
    this.resetError();
    if (this.sudokuService.isCorrect(this.getGrid())) {
      this.publish();
      return;
    }
    this.nbIndice += 1;
    let error: number[][] = this.sudokuService.validationGrille(this.getGrid());
    for (let y=0; y<error.length; y++) {
      for (let x=0; x<error[y].length; x++) {
        if (error[y][x] == -1) {
          this.marqueError(x,y);
        }
      }
    }
  }

  private getGrid():number[][] {
    let grid: number[][] = [];
    for (let y=0; y<9; y++) {
      grid.push([]);
      for (let x=0; x<9; x++) {
        grid[y].push(Number((document.getElementById("xy"+y.toString()+x.toString()) as HTMLInputElement).value) | 0);
      }
    }
    return grid;
  }

  private marqueError(x:number,y:number) {
    let error = document.getElementById("xy"+y.toString()+x.toString());
    if (error == null) return;
    error.classList.add("faux");
  }

  private resetError() {
    let errors = document.getElementsByClassName("faux");
    for (let i=0; i<errors.length; i++) {
      errors[i].classList.remove("faux");
    }
  }

  private Request(difficulty: string) {
    switch (difficulty) {
      case 'easy':
        this.grille = this.sudokuService.getEasy();
        break;
      case 'medium':
        this.grille = this.sudokuService.getMedium();
        break;
      case 'hard':
        this.grille = this.sudokuService.getHard();
        break;
      default:
        this.grille = this.sudokuService.getEasy();
        break;
    }
    if (this.grille.length == 0) {
      setTimeout(()=> {this.Request(difficulty)},100);
      return;
    }
    this.updateNb();
  }

  public colorSameRowCol(a:number ,b:number, event: Event) {
    if (event.target != null) this.lastInputClicked = <HTMLInputElement>event.target;
    this.resetColor();
    let xp:number = (a%3)*3+ b%3;
    let yp:number = Math.floor(a/3)*3+Math.floor(b/3);
    for(let x:number=0; x<9; x++) {
      for(let y:number=0; y<9; y++) {
        if (xp == x || yp == y) {
          let newx:number = (y%3)*3+(x%3);
          let newy:number = Math.floor(y/3)*3+Math.floor(x/3);
          document.getElementById("xy"+newx.toString()+newy.toString())?.classList.add("selectedRowCol")
        }
      }
    }
  }

  public resetColor() {
    this.updateNb();
    let inputs:HTMLCollection = document.getElementsByClassName("selectedRowCol");
    for (let i:number=0;inputs.length>0;i++) {
      inputs[0].classList.remove("selectedRowCol");
    }
  }

  private updateNb() {
    for(let i:number=1;i<10;i++) {
      this.nbEachNumber[i-1] = this.nb(i);
    }
  }

  private nb(i:number) {
    let a:number=0
    this.grille.forEach(e=>{
      a += e.filter((v) => (v == i)).length
    })
    return 9-a;
  }

  public key() {
    this.grille = this.getGrid();
    this.updateNb();
  }

  public addNumber(nb:number) {
    this.resetColor();
    if (this.nbEachNumber[nb-1] <= 0) return;
    if (this.lastInputClicked == null) return;
    this.lastInputClicked.value = nb.toString();
    this.key();
  }
}